import os
import asyncio
from uuid import uuid1
from tabulate import tabulate
from aiofile import AIOFile
import aiohttp

from . import client
from .gameCrafterSession import GameCrafterSession

baseUrl = "https://www.thegamecrafter.com"

async def createGame(gameCrafterSession, name, designerId):
    game = await client.postGame(gameCrafterSession, name, designerId)
    
    gameName = game["name"]
    gameId = game["id"]
    editUrl = "%s%s%s" % (baseUrl, "/publish/editor/", gameId)
    print("Created %s. Edit it here %s" % (gameName, editUrl))

    return game

async def createFolderAtRoot(gameCrafterSession, name):
    user = await client.getUser(gameCrafterSession)
    return await client.postFolder(gameCrafterSession, name, user['root_folder_id'])

async def createFolderAtParent(gameCrafterSession, name, folderId):
    return await client.postFolder(gameCrafterSession, name, folderId)

async def uploadFile(gameCrafterSession, filepath, folderId):
    if not os.path.isfile(filepath):
        raise Exception ('Not a file: %s' % filepath)
    
    filename = os.path.basename(filepath)

    with open(filepath, "rb") as fileToUpload:
        return await client.postFile(gameCrafterSession, fileToUpload, filename, folderId)

async def createPokerDeck(gameCrafterSession, name, quantity, gameId, imageFileId):
    return await client.postPokerDeck(gameCrafterSession, name, quantity, gameId, imageFileId)

async def createPokerCard(gameCrafterSession, name, deckId, quantity, imageFileId):
    return await client.postPokerCard(gameCrafterSession, name, deckId, quantity, imageFileId)

async def createSmallStoutBox(gameCrafterSession, gameId, name, quantity, topImageFileId, bottomImageFileId):
    return await client.postSmallStoutBox(gameCrafterSession, gameId, name, quantity, topImageFileId, bottomImageFileId)

async def createDocument(gameCrafterSession, name, quantity, gameId, pdfFileId):
    return await client.postDocument(gameCrafterSession, name, quantity, gameId, pdfFileId)

async def printUser(gameCrafterSession):
    print(await client.getUser(gameCrafterSession))

async def listGames(gameCrafterSession):
    gamesResponse = await client.getGamesForUser(gameCrafterSession)
    await printGames(gamesResponse["items"])

async def listGamesForUserDesigners(gameCrafterSession):
    designersResponse = await client.getDesigners(gameCrafterSession)
    designers = designersResponse["items"]

    games = []
    for designer in designers:
        gamesResponse = await client.getGamesForDesignerId(gameCrafterSession, designer["id"])
        games.extend(gamesResponse["items"])

    printGames(games)   

async def printGames(clientSession, games):
    headers = ["name", "id", "link"]
    data = []

    for game in games:
        gameName = game["name"]
        gameId = game["id"]
        gameLink = os.path.join(baseUrl, game["edit_uri"])
        data.append([gameName, gameId, gameLink])

    print(tabulate(data, headers=headers, tablefmt='orgtbl'))

async def listDesigners(gameCrafterSession):
    designersResponse = await client.getDesigners(gameCrafterSession)

    headers = ["name", "id"]
    data = []
    for designer in designersResponse["items"]:
        print(designer)
        name = designer["name"]
        designerId = designer["id"]
        data.append([name, designerId])

    print(tabulate(data, headers=headers, tablefmt='orgtbl'))

async def login():
    
    publicApiKey = os.environ.get('THEGAMECRAFTER_PUBLIC_KEY')
    if not publicApiKey:
        raise Exception('Could not log in. You need to set the env variable THEGAMECRAFTER_PUBLIC_KEY. Value is %s' % publicApiKey)
    
    userName = os.environ.get('THEGAMECRAFTER_USER')
    if not userName:
        raise Exception('Could not log in. You need to set the env variable THEGAMECRAFTER_USER. Value is %s' % userName)

    userPassword = os.environ.get('THEGAMECRAFTER_PASSWORD')
    if not userPassword:
        raise Exception('Could not log in. You need to set the env variable THEGAMECRAFTER_PASSWORD. Value is %s' % userPassword)
    
    gameCrafterSession = GameCrafterSession(aiohttp.ClientSession())
    login = await client.login(gameCrafterSession, publicApiKey, userName, userPassword)
    gameCrafterSession.login(sessionId=login["id"], userId=login["user_id"])
    return gameCrafterSession

async def logout(gameCrafterSession):
    await client.logout(gameCrafterSession)


import os
import asyncio
from datetime import datetime
from . import httpClient

gameCrafterBaseUrl = "https://www.thegamecrafter.com/api"

async def login(gameCrafterSession, publicApiKey, userName, userPassword):
    url = "%s/session" % gameCrafterBaseUrl
    return await httpClient.post(gameCrafterSession, url, 
        api_key_id = publicApiKey,
        username = userName,
        password = userPassword
    )

async def logout(gameCrafterSession):
    url = "%s/session/%s" % (gameCrafterBaseUrl, gameCrafterSession.sessionId)
    await httpClient.delete(gameCrafterSession, url)
    await gameCrafterSession.httpSession.close() 

async def getUser(gameCrafterSession):
    url = "%s/user/%s" % (gameCrafterBaseUrl, gameCrafterSession.userId)
    
    return await httpClient.get(gameCrafterSession, url, 
        session_id = gameCrafterSession.sessionId
    )

async def getDesigners(gameCrafterSession):
    url = "%s/user/%s/designers" % (gameCrafterBaseUrl, gameCrafterSession.userId)
    return await httpClient.get(gameCrafterSession, url,
        session_id = gameCrafterSession.sessionId
    )

async def getGamesForDesignerId(gameCrafterSession, designerId):
    url = "%s/designer/%s/games" % (gameCrafterBaseUrl, designerId)
    return await httpClient.get(gameCrafterSession, url,
        session_id = gameCrafterSession.sessionId
    )

async def getGamesForUser(gameCrafterSession):
    url = "%s/user/%s/games" % (gameCrafterBaseUrl, gameCrafterSession.userId)
    return await httpClient.get(gameCrafterSession, url,
        session_id = gameCrafterSession.sessionId
    )

async def postGame(gameCrafterSession, name, designerId):
    url = "%s/game" % gameCrafterBaseUrl
    return await httpClient.post(gameCrafterSession, url,
        session_id = gameCrafterSession.sessionId,
        name = name,
        designer_id = designerId,
        description='Automatically created (%s)' % name,
    )

async def postPokerDeck(gameCrafterSession, name, quantity, gameId, backImageFileId):
    url = "%s/pokerdeck" % gameCrafterBaseUrl
    return await httpClient.post(gameCrafterSession, url,
        session_id = gameCrafterSession.sessionId,
        name = name,
        game_id = gameId,
        quantity = quantity,
        back_id = backImageFileId,
        has_proofed_back = 1
    )

async def postPokerCard(gameCrafterSession, name, deckId, quantity, imageFileId):
    url = "%s/pokercard" % gameCrafterBaseUrl
    return await httpClient.post(gameCrafterSession, url,
        session_id = gameCrafterSession.sessionId,
        name = name,
        deck_id = deckId,
        quantity = quantity,
        face_id = imageFileId,
        back_from = "Deck",
        has_proofed_face = 1,
        has_proofed_back = 1
    )

async def postSmallStoutBox(gameCrafterSession, gameId, name, quantity, topImageFileId, backImageFileId):
    url = "%s/smallstoutbox" % gameCrafterBaseUrl
    return await httpClient.post(gameCrafterSession, url,
        session_id = gameCrafterSession.sessionId,
        name = name,
        game_id = gameId,
        quantity = quantity,
        top_id = topImageFileId,
        has_proofed_top = 1,
        bottom_id = backImageFileId,
        has_proofed_bottom = 1
    )

async def postDocument(gameCrafterSession, name, quantity, gameId, pdfFileId):
    url = "%s/document" % gameCrafterBaseUrl
    return await httpClient.post(gameCrafterSession, url,
        session_id = gameCrafterSession.sessionId,
        name = name,
        game_id = gameId,
        quantity = quantity,
        pdf_id = pdfFileId,
    )

async def postFolder(gameCrafterSession, name, folderParentId):
    url = "%s/folder" % gameCrafterBaseUrl
    return await httpClient.post(gameCrafterSession, url,
        session_id = gameCrafterSession.sessionId,
        name=name,
        user_Id=gameCrafterSession.userId,
        parent_id=folderParentId,
    )

async def postFile(gameCrafterSession, file, filename, folderId):
    url = "%s/file" % gameCrafterBaseUrl
    return await httpClient.post(gameCrafterSession, url,
        session_id = gameCrafterSession.sessionId,
        file=file, 
        name=filename, 
        folder_id=folderId)








import aiohttp
import asyncio
class GameCrafterSession:
    def __init__ (self, httpSession):
        self.httpSession = httpSession

    def login(self, sessionId, userId):
        self.sessionId = sessionId
        self.userId = userId


import os
import json

async def post(gameCrafterSession, url, **kwargs):
    async with gameCrafterSession.httpSession.post(url, data=kwargs) as response:
        return await handleResponse(url, response)

async def get(gameCrafterSession, url, **kwargs):
    async with gameCrafterSession.httpSession.get(url, params=kwargs) as response:
        return await handleResponse(url, response)

async def delete(gameCrafterSession, url, **kwargs):
    async with gameCrafterSession.httpSession.delete(url, params=kwargs) as response:
        return await handleResponse(url, response)

async def handleResponse(url, response):
    statusCode = str(response.status)
    responseText = await response.text()
    responseJson = json.loads(responseText)
    if not statusCode.startswith('2'):
        print ('FAIL', response)
        print ('FAIL', responseJson)
        raise Exception('%s Returned %s.' % (url, statusCode))
    
    return responseJson['result']


from setuptools import setup, find_packages

setup(
    name = "gamecrafterclient",
    version = "1.0.10",
    author = "Oliver Barnum",
    author_email = "oliverbarnum32@gmail.com",
    description = "Python API client for the game crafter.",
    url = "https://github.com/TheNextGuy32/gameCrafterClient",
    packages=find_packages(),
    install_requires=["tabulate", "asyncio","aiofile","aiohttp"],
    download_url="https://github.com/TheNextGuy32/gameCrafterClient/archive/1.0.4.tar.gz",
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
)